from pathlib import Path

import requests


class File:
    config = None

    def __init__(self, client=None):
        self.config = client.get_config()

    def upload_application(self, file_category=None, project_name=None, file_path=None):
        path_object = Path(file_path)
        filename = path_object.name
        data = {
            "fileName": filename,
            "fileCategory": file_category,
            "userName": self.config.get("username"),
            "projectName": project_name
        }

        md5 = self.get_file_md5(filepath=file_path)
        data["md5sum"] = md5
        try:
            file_object = open(file_path, 'rb')
            files = {'file': file_object}
            response_message = self.__upload(data=data, files=files)
            file_object.close()
            return response_message
        except FileNotFoundError:
            return "Error: No such file or directory: " + filename

    def get_application_info(self, file_name=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        new_params = {
            "fileName": file_name
        }
        file_api_url = self.config.get("api_url") + "testexecute/files"
        response = requests.get(file_api_url, params=new_params, headers=new_headers)

        file_list = response.json()['data']['list']
        return_message = {}
        if len(file_list) == 0:
            return "Failure: File with name `" + file_name + "` not found."
        elif len(file_list) == 1:
            file_category = file_list[0]['fileCategory']
            try:
                packageName = file_list[0]['fileParameters']['packageName']
            except:
                packageName = ""
            if file_category == 'android-application':
                return_message = {"fileName": file_name,
                                  "fileCategory": file_list[0]['fileCategory'],
                                  "md5": file_list[0]['meta']['md5sum'],
                                  "fileURL": file_list[0]['meta']['s3Url'],
                                  "fileUUID": file_list[0]['uuid'],
                                  "packageName": packageName
                                  }
            elif file_category == 'ios-application':
                return_message = {"fileName": file_name,
                                  "fileCategory": file_list[0]['fileCategory'],
                                  "md5": file_list[0]['meta']['md5sum'],
                                  "fileURL": file_list[0]['meta']['s3Url'],
                                  "fileUUID": file_list[0]['uuid']
                                  }
            elif file_category == 'android-test-application':
                return_message = {"fileName": file_name,
                                  "fileCategory": file_list[0]['fileCategory'],
                                  "md5": file_list[0]['meta']['md5sum'],
                                  "fileURL": file_list[0]['meta']['s3Url'],
                                  "fileUUID": file_list[0]['uuid'],
                                  "testCodePackageName": file_list[0]['fileParameters']['testCodePackageName'],
                                  "testRunnerName": file_list[0]['fileParameters']['testRunnerName']
                                  }
            elif file_category == 'ios-test-application':
                return_message = {"fileName": file_name,
                                  "fileCategory": file_list[0]['fileCategory'],
                                  "md5": file_list[0]['meta']['md5sum'],
                                  "fileURL": file_list[0]['meta']['s3Url'],
                                  "fileUUID": file_list[0]['uuid'],
                                  "XCTestRunFileUrl": file_list[0]['fileParameters']['xctestrunFileUrl']
                                  }
            return return_message

    def get_application_list(self, file_category=None, project_name=None):
        status_message = self.get_file_info_list(file_category=file_category, project_name=project_name)
        status_message_filtered = []
        try:
            for f in status_message:
                if f["fileCategory"] == "android-application" or f["fileCategory"] == "ios-application":
                    status_message_filtered.append(f)
            return status_message_filtered
        except TypeError:
            return "Error: Project Name or File Name doesn't exist"

    def get_application_list_all(self):
        status_message = self.get_file_info_list_all()
        return status_message
    # Native Test Application

    def upload_native_test_application(self, file_category=None, project_name=None, file_path=None):
        status_message = self.upload_application(self, file_category=file_category,
                                                 project_name=project_name,
                                                 file_path=file_path)
        return status_message

    def get_native_test_application_info(self, file_name=None):
        status_message = self.get_application_info(file_name=file_name)
        return status_message

    def get_native_test_application_list(self, file_category=None, project_name=None):
        status_message = self.get_file_info_list(file_category=file_category, project_name=project_name)
        status_message_filtered = []
        try:
            for f in status_message:
                if f["fileCategory"] == "android-test-application" or f["fileCategory"] == "ios-test-application":
                    status_message_filtered.append(f)
            return status_message_filtered
        except TypeError:
            return "Error: Project Name Doesn't exist"

    def get_file_md5(self, filepath=None):
        import hashlib
        md5_hash = ''
        try:
            with open(filepath, "rb") as f:
                file_bytes = f.read()  # read file as bytes
                readable_hash = hashlib.md5(file_bytes).hexdigest()
                md5_hash = readable_hash
            return md5_hash
        except FileNotFoundError:
            return "Error: No such file or directory "

    def __upload(self, data=None, files=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}

        file_api_url = self.config.get("api_url") + "testexecute/files"
        file_name = data["fileName"]
        # Leg 1 - get the s3 file upload URL
        response = requests.post(file_api_url, json=data, headers=new_headers)

        if response.status_code == 200:
            if response.json()['status'] == 409:
                return "Error: File `" + file_name + "` already exists."
        else:
            return "Error: " + response.text

        s3_file_upload_url = response.json()['data']['uploadUrl']

        # print("\n s3 link: ", s3_file_upload_url)
        # Leg 2 - upload the file using s3 upload URL
        response = requests.put(s3_file_upload_url, files=files)
        # print("\n s3 response: ", response)
        if response.status_code == 200:
            return "Success: File `" + file_name + "` uploaded successfully."
        else:
            return "Failure: File `" + file_name + "` not uploaded."

    def get_file_info_list(self, file_category=None, project_name=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}

        if project_name is None:
            project_name = ""

        new_params = {
            "fileCategory": file_category,
            "projectName": project_name,
            "fileStatus": "processed"
        }
        file_api_url = self.config.get("api_url") + "testexecute/files"
        # Fetch list of files uploaded
        response = requests.get(file_api_url, params=new_params, headers=new_headers)

        file_list = response.json()['data']['list']
        return_message = []

        if len(file_list) > 0:
            for f in file_list:
                file_category = f['fileCategory']
                if file_category == 'android-application':
                    file_info = {"fileName": f['fileName'],
                                 "fileCategory": f['fileCategory'],
                                 "md5": f['meta']['md5sum'],
                                 "fileURL": f['meta']['s3Url'],
                                 "fileUUID": f['uuid'],
                                 "packageName": f['fileParameters']['packageName'],
                                 "projectName": project_name
                                 }
                    return_message.append(file_info)
                elif file_category == 'ios-application':
                    file_info = {"fileName": f['fileName'],
                                 "fileCategory": f['fileCategory'],
                                 "md5": f['meta']['md5sum'],
                                 "fileURL": f['meta']['s3Url'],
                                 "fileUUID": f['uuid'],
                                 "projectName": project_name
                                 }
                    return_message.append(file_info)
                elif file_category == 'android-test-application':
                    file_info = {"fileName": f['fileName'],
                                 "fileCategory": f['fileCategory'],
                                 "md5": f['meta']['md5sum'],
                                 "fileURL": f['meta']['s3Url'],
                                 "fileUUID": f['uuid'],
                                 "testCodePackageName": f['fileParameters']['testCodePackageName'],
                                 "testRunnerName": f['fileParameters']['testRunnerName'],
                                 "projectName": project_name
                                 }
                    return_message.append(file_info)
                elif file_category == 'ios-test-application':
                    file_info = {"fileName": f['fileName'],
                                 "fileCategory": f['fileCategory'],
                                 "md5": f['meta']['md5sum'],
                                 "fileURL": f['meta']['s3Url'],
                                 "fileUUID": f['uuid'],
                                 "XCTestRunFileUrl": f['fileParameters']['xctestrunFileUrl'],
                                 "projectName": project_name
                                 }
                    return_message.append(file_info)
            return return_message

    def get_file_info_list_all(self):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}

        file_api_url = self.config.get("api_url") + "testexecute/files"
        # Fetch list of files uploaded
        response = requests.get(file_api_url, headers=new_headers)

        file_list = response.json()['data']['list']
        return_message = []

        if len(file_list) > 0:
            for f in file_list:
                file_category = f['fileCategory']
                if file_category == 'android-application':
                    file_info = {"fileName": f['fileName'],
                                 "fileCategory": f['fileCategory'],
                                 "md5": f['meta']['md5sum'],
                                 "fileURL": f['meta']['s3Url'],
                                 "fileUUID": f['uuid'],
                                 "projectName": ""
                                 }
                    return_message.append(file_info)
                elif file_category == 'ios-application':
                    file_info = {"fileName": f['fileName'],
                                 "fileCategory": f['fileCategory'],
                                 "md5": f['meta']['md5sum'],
                                 "fileURL": f['meta']['s3Url'],
                                 "fileUUID": f['uuid'],
                                 "projectName": ""
                                 }
                    return_message.append(file_info)
                elif file_category == 'android-test-application':
                    file_info = {"fileName": f['fileName'],
                                 "fileCategory": f['fileCategory'],
                                 "md5": f['meta']['md5sum'],
                                 "fileURL": f['meta']['s3Url'],
                                 "fileUUID": f['uuid'],
                                 "testCodePackageName": f['fileParameters']['testCodePackageName'],
                                 "testRunnerName": f['fileParameters']['testRunnerName'],
                                 "projectName": ""
                                 }
                    return_message.append(file_info)
                elif file_category == 'ios-test-application':
                    file_info = {"fileName": f['fileName'],
                                 "fileCategory": f['fileCategory'],
                                 "md5": f['meta']['md5sum'],
                                 "fileURL": f['meta']['s3Url'],
                                 "fileUUID": f['uuid'],
                                 "XCTestRunFileUrl": f['fileParameters']['xctestrunFileUrl'],
                                 "projectName": ""
                                 }
                    return_message.append(file_info)
            return return_message

    def delete_file(self, file_name=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}

        file_info = self.get_application_info(file_name=file_name)
        try:
            file_api_url = self.config.get("api_url") + "testexecute/files?fileId=" + file_info["fileUUID"]
        except TypeError:
            return "Failure: File `" + file_name + "` not available"
        response = requests.delete(file_api_url, headers=new_headers)
        try:
            if response.status_code == 200:
                return "Success: File `" + file_name + "` deleted successfully."
            else:
                return "Failure: File `" + file_name + "` not deleted."
        except TypeError:
            return "Failure: File `" + file_name + "` not delete"



import requests


class User:
    config = None

    def __init__(self, client=None):
        self.config = client.get_config()

    def login(self):
        new_headers = {'X-Amz-Target': 'AWSCognitoIdentityProviderService.InitiateAuth',
                       'Content-Type': 'application/x-amz-json-1.1'}
        login_url = "https://cognito-idp.ap-south-1.amazonaws.com/"
        data = {
            "AuthParameters": {
                "USERNAME": self.config.get("username"),
                "PASSWORD": self.config.get("password")
            },
            "AuthFlow": "USER_PASSWORD_AUTH",
            "ClientId": self.config.get("client_id")
        }

        resp = requests.post(login_url, json=data, headers=new_headers)
        api_access_token = resp.json()['AuthenticationResult']['IdToken']
        return api_access_token

    def logout(self):
        pass




class Schedule:
    devices = None
    test_configuration = {
        "captureHAR": False,
        "captureCPUMetrics": False,
        "captureMemoryMetrics": False,
        "captureBatteryMetrics": False,
        "captureGraphicsMetrics": False,
        "captureDeviceScreenShots": False,
        "recordDeviceScreen": False,
        "captureDeviceNetworkPackets": False,
        "captureAutomationLogs": False,
        "captureSystemDebugLogs": False,
        "captureLiveLogs": False
    }
    schedule_configuration = {}
    test_action = {}
    test_parameters = {}
    application_url = None
    test_application_url = None
    executionType = "NOW"

    def __init__(self):
        pass

    def set_test_configuration(self, test_configuration = None):
        pass







import json
from pathlib import Path
import requests


class TestExecute:
    config = None
    client = None

    def __init__(self, client=None):
        self.config = client.get_config()
        self.client = client

    def execute_test_now(self,
                         project_name=None,
                         test_framework=None,
                         application_file_name=None,
                         test_application_file_name=None,
                         devices=None,
                         test_configuration=None,
                         test_parameters=None
                         ):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        test_parameters_req = {
            "testType": "app-automation",
            "maxTestDuration": test_parameters["maxTestDuration"],
            "testFramework": test_framework,
            "projectName": project_name,
            "packageName": ""
        }
        application_url = self.client.get_application_info(file_name=application_file_name)["fileURL"]
        test_application_url = self.client.get_native_test_application_info(file_name=test_application_file_name)[
            "fileURL"]

        data = {
            "deviceId": devices,
            "testConfiguration": test_configuration,
            "scheduleConfiguration": {},
            "testAction": {
                "pre": {},
                "post": {}
            },
            "testParameters": test_parameters_req,
            "applicationUrl": application_url,
            "testApplicationUrl": test_application_url,
            "executionType": "NOW"
        }

        print(data)

        test_api_url = self.config.get("api_url") + "testexecute/schedules"
        response = requests.post(test_api_url, json=data, headers=new_headers)
        if response.status_code == 200:
            try:
                schedule_id = {
                    "scheduleId": response.json()["data"]["scheduleId"]
                }
                test_api_url = self.config.get("api_url") + "testexecute/schedules"
                response = requests.get(test_api_url, params=schedule_id, headers=new_headers)
                print(response.json())
                schedule_details = response.json()['data']['list'][0]['testExecutions']
                testIds = []
                for tests in schedule_details:
                    testIds.append(tests['uuid'])
                response = {
                    "message": "Success: Executed/Scheduled successfully",
                    "testId": testIds
                }
                return response
            except Exception as e:
                print(e)
                return {"statusCode:": response.status_code, "message": response.text}

        else:
            return {"statusCode:": response.status_code, "message": response.text}

    def get_test_info(self, test_id=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        new_params = {
        }
        test_api_url = self.config.get("api_url") + "analytics/tests/" + test_id + "/info"
        # Fetch info of test
        response = requests.get(test_api_url, params=new_params, headers=new_headers)
        if response.status_code == 200:
            test_list = json.loads(response.text)
            test_list = test_list['body']
            formatted_response = {
                "projectName": test_list['projectName'],
                # "testFramework": test_list['extra']['testType'],
                "applicationFileName": test_list['appVersion'],
                "testApplicationFileName": test_list['scriptName'],
                "device": test_list['deviceName'],
                "testStartTime": test_list['testStartTime'],
                "testEndTime": test_list['testEndTime'],
                "testUUID": test_list['uuid']['testId'],
                "testStatus": test_list['testStatus']
                # "testStatusDescription": ""
            }
            return formatted_response
        else:
            return {"statusCode:": response.status_code, "message": response.text}

    def schedule_test_executions(self,
                                 project_name=None,
                                 test_framework=None,
                                 application_file_name=None,
                                 test_application_file_name=None,
                                 devices=None,
                                 test_configuration=None,
                                 test_parameters=None,
                                 start_date_time=None,
                                 end_date_time=None,
                                 interval=None
                                 ):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        test_parameters_req = {
            "testType": "app-automation",
            "maxTestDuration": test_parameters["maxTestDuration"],
            "testFramework": test_framework,
            "projectName": project_name,
            "packageName": ""
        }
        application_url = self.client.get_application_info(file_name=application_file_name)["fileURL"]
        test_application_url = self.client.get_native_test_application_info(file_name=test_application_file_name)[
            "fileURL"]

        data = {
            "deviceId": devices,
            "testConfiguration": test_configuration,
            "scheduleConfiguration": {
                "startTime": start_date_time,
                "endTime": end_date_time,
                "interval": interval
            },
            "testAction": {
                "pre": {},
                "post": {}
            },
            "testParameters": test_parameters_req,
            "applicationUrl": application_url,
            "testApplicationUrl": test_application_url,
            "executionType": "SCHEDULE"
        }

        print(data)
        test_api_url = self.config.get("api_url") + "testexecute/schedules"
        response = requests.post(test_api_url, json=data, headers=new_headers)
        if response.status_code == 200:
            try:
                schedule_id = {
                    "scheduleId": response.json()["data"]["scheduleId"]
                }
                test_api_url = self.config.get("api_url") + "testexecute/schedules"
                response = requests.get(test_api_url, params=schedule_id, headers=new_headers)
                print(response.json())
                scheduleId = response.json()['data']['list'][0]['uuid']
                testIds = response.json()['data']['list'][0]['testExecutions']
                testId_list = []
                for tests in testIds:
                    test_new = {"testUUID": tests['uuid'], "testStartDateTime": tests['testScheduledTime']}
                    testId_list.append(test_new)

                formatted_response = {
                    "message": "Success: Executed/Scheduled successfully",
                    "scheduleUUID": scheduleId,
                    "testRuns": testId_list
                }
                return formatted_response
            except Exception as e:
                print(e.with_traceback())
                return {"statusCode:": response.status_code, "message": response.text}
        else:
            return {"statusCode:": response.status_code, "message": response.text}

    def get_test_schedule_info(self, schedule_id=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        schedule_id = {
            "scheduleId": schedule_id
        }
        test_api_url = self.config.get("api_url") + "testexecute/schedules"
        response = requests.get(test_api_url, params=schedule_id, headers=new_headers)
        print(response.json())
        try:
            response = response.json()['data']['list'][0]
            testInfo = []
            for tests in response['testExecutions']:
                test_details = {
                    "device": tests['devices'][0],
                    "testStartTime": tests['testScheduledTime'],
                    "testEndTime": "",
                    "testUUID": tests['uuid'],
                    "testStatus": tests['testStatus'],
                    "testStatusDescription": ""
                }
                testInfo.append(test_details)

            formatted_response = {
                "scheduleUUID": response['uuid'],
                "scheduleStartTime": response['scheduleConfiguration']['startTime'],
                "scheduleEndTime": response['scheduleConfiguration']['endTime'],
                "testInterval": response['scheduleConfiguration']['interval'],
                "testConfiguration": response['testExecutions'][0]['testConfiguration'],
                "testParameters": response['testExecutions'][0]['testParameters'],
                "projectName": response['testExecutions'][0]['testParameters']['projectName'],
                "testFramework": response['testExecutions'][0]['testParameters']['testFramework'],
                "applicationFileName": response['testExecutions'][0]['applicationUrl'],
                "testApplicationFileName": response['testExecutions'][0]['testApplicationUrl'],
                "testInfo": testInfo
            }

            return formatted_response
        except IndexError:
            return "Failure: schedule id " + schedule_id["scheduleId"] + " doesn't exists."

    def get_test_schedule_list(self, from_date_time=None, to_date_time=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        params = {
            #        "startTime": from_date_time,
            #        "endTime": to_date_time
        }
        test_api_url = self.config.get("api_url") + "testexecute/schedules"
        response = requests.get(test_api_url, params=params, headers=new_headers)
        complete_response = []
        for res in response.json()['data']['list']:
            testInfo = []
            for tests in res['testExecutions']:
                test_details = {
                    "device": tests['devices'][0],
                    "testStartTime": tests['testScheduledTime'],
                    "testEndTime": "",
                    "testUUID": tests['uuid'],
                    "testStatus": tests['testStatus'],
                    "testStatusDescription": ""
                }
                testInfo.append(test_details)
            formatted_response = {
                "scheduleUUID": res['uuid'],
                "scheduleStartTime": res['scheduleConfiguration']['startTime'],
                "scheduleEndTime": res['scheduleConfiguration']['endTime'],
                "testInterval": res['scheduleConfiguration']['interval'],
                "testConfiguration": res['testExecutions'][0]['testConfiguration'],
                "testParameters": res['testExecutions'][0]['testParameters'],
                "projectName": res['testExecutions'][0]['testParameters']['projectName'],
                "testFramework": res['testExecutions'][0]['testParameters']['testFramework'],
                "applicationFileName": res['testExecutions'][0]['applicationUrl'],
                "testApplicationFileName": res['testExecutions'][0]['testApplicationUrl'],
                "testInfo": testInfo
            }
            complete_response.append(formatted_response)
        return complete_response

    def execute_test(self, client=None, device_list=None, test_configuration={}, schedule_configuration={},
                     test_parameters={},
                     execution_type=None, application_url=None, application_test_url=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        data = {
            "deviceId": device_list,
            "testConfiguration": test_configuration,
            "scheduleConfiguration": schedule_configuration,
            "testAction": {
                "pre": {},
                "post": {}
            },
            "testParameters": test_parameters,
            "applicationUrl": application_url,
            "testApplicationUrl": application_test_url,
            "executionType": execution_type
        }
        test_api_url = self.config.get("api_url") + "testexecute/schedules"
        response = requests.post(test_api_url, json=data, headers=new_headers)
        if response.status_code == 200:
            my_resp = json.loads(response.text)
            my_resp = my_resp['data']
            return my_resp
        else:
            return {"statusCode:": response.status_code, "message": response.text}
        # return response.status_code, response.text

    def list_schedules(self, client=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        new_params = {
        }
        test_api_url = self.config.get("api_url") + "testexecute/schedules"
        # Fetch list of schedules
        response = requests.get(test_api_url, params=new_params, headers=new_headers)
        if response.status_code == 200:
            my_resp = json.loads(response.text)
            my_resp = my_resp['data']['list']
            return my_resp
        else:
            return {"statusCode:": response.status_code, "message": response.text}
        # return response.status_code, response.text

    def delete_schedule(self, schedule_id=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        new_params = {
            "scheduleId": schedule_id
        }
        test_api_url = self.config.get("api_url") + "testexecute/schedules"
        # Delete schedule
        response = requests.delete(test_api_url, params=new_params, headers=new_headers)
        if response.status_code == 200:
            my_resp = json.loads(response.text)
            my_resp = my_resp["message"]
            return my_resp
        else:
            return {"statusCode:": response.status_code, "message": response.text}
        # return response.text

    def abort_test(self, test_id=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        data = {
            "testId": test_id
        }
        test_api_url = self.config.get("api_url") + "testexecute/tests"
        # abort test
        response = requests.put(test_api_url, json=data, headers=new_headers)
        return response.text

    def schedule_test(self, client=None, device_list=None, schedule_configuration={}, test_configuration={},
                      test_parameters={}, application_url=None, application_test_url=None):
        # schedule_configuration = {
        #     "startTime": start_time,
        #     "endTime": end_time,
        #     "interval": interval
        # }
        # test_parameters = {
        #     "maxTestDuration": max_duration,
        #     "testFramework": test_framework,
        #     "projectName": project_name
        # }
        execution_type = "SCHEDULE"
        status_message = self.execute_test(device_list=device_list, test_configuration=test_configuration,
                                           schedule_configuration=schedule_configuration,
                                           test_parameters=test_parameters, execution_type=execution_type,
                                           application_url=application_url,
                                           application_test_url=application_test_url)
        return status_message

    def test_now(self, client=None, device_list=None, test_configuration={}, test_parameters={},
                 application_url=None, application_test_url=None):
        schedule_configuration = {}
        # test_parameters = {
        #     "maxTestDuration": max_duration,
        #     "testFramework": test_framework,
        #     "projectName": project_name
        # }
        execution_type = "NOW"
        status_code, status_message = self.execute_test(device_list=device_list, test_configuration=test_configuration,
                                                        schedule_configuration=schedule_configuration,
                                                        test_parameters=test_parameters, execution_type=execution_type,
                                                        application_url=application_url,
                                                        application_test_url=application_test_url)
        return status_message

    def update_schedule(self, data=None, schedule_id=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        schedule_id = {
            "uuid": schedule_id
        }
        test_api_url = self.config.get("api_url") + "testexecute/edit/schedule"
        response = requests.put(test_api_url, json=data, params=schedule_id, headers=new_headers)
        print(response.json())

    def update_test(self, data=None, test_id=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        test_id = {
            "uuid": test_id
        }
        test_api_url = self.config.get("api_url") + "testexecute/edit/test"
        response = requests.put(test_api_url, json=data, params=test_id, headers=new_headers)
        print(response.json())



import json

import requests


class Project:
    config = None

    def __init__(self, client=None):
        self.config = client.get_config()

    def create_project(self, project_name=None, project_description=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        data = {
            "name": project_name,
            "description": project_description
        }
        project_api_url = self.config.get("api_url") + "testexecute/projects"
        response = requests.post(project_api_url, json=data, headers=new_headers)
        if response.json()["status"] == 200 and response.json()["message"] == "Success":
            return "Success"
        elif response.json()["status"] == 409 and response.json()["message"] == "Project already exists":
            return "Failure: Project with "+project_name+" already exists."

    def get_project_info(self, project_name=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        new_params = {
            "name": project_name
        }
        project_api_url = self.config.get("api_url") + "testexecute/projects"
        # Fetch list of projects
        response = requests.get(project_api_url, params=new_params, headers=new_headers)
        project_list = response.json()["data"]["list"]
        return_message = {}

        if response.status_code == 200 and len(project_list) == 1:
            response_project_name = project_list[0]["name"]
            if response_project_name == project_name:
                response_project_description = project_list[0]["description"]
                response_project_uuid = project_list[0]["uuid"]
                return_message["projectName"] = response_project_name
                return_message["projectDescription"] = response_project_description
                return_message["projectUUID"] = response_project_uuid
                return return_message
        elif response.status_code == 200 and len(project_list) == 0:
            return "Failure: Project with name `" + project_name + "` not found."

    def delete_project(self, project_name=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        project_info = self.get_project_info(project_name=project_name)
        if project_info == "Failure: Project with name `" + project_name + "` not found.":
            return project_info
        else:
            delete_project_url = self.config.get("api_url") + "testexecute/projects?projectId=" + project_info["projectUUID"]
            response = requests.delete(delete_project_url, headers=new_headers)
            if response.json()["status"] == 200 and response.json()["message"] == "Success":
                return "Success"

    def get_project_list(self):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}

        project_api_url = self.config.get("api_url") + "testexecute/projects"
        # Fetch list of projects
        response = requests.get(project_api_url, headers=new_headers)
        project_list = response.json()["data"]["list"]

        response_project_list = []
        if response.status_code == 200 and len(project_list) > 0:
            for p in project_list:
                project_info = {"projectName": p["name"],
                                "projectDescription": p["description"],
                                "projectUUID": p["uuid"]}
                response_project_list.append(project_info)
            return response_project_list
        elif response.status_code == 200 and len(project_list) == 0:
            return "Failure: Project list is empty."



import configparser
from pathlib import Path

from mozark_sdk.user import User
from mozark_sdk.project import Project
from mozark_sdk.file import File
from mozark_sdk.device import Device
from mozark_sdk.tray import Tray
from mozark_sdk.executetest import TestExecute
from mozark_sdk.test_analytics import TestAnalytics


class Client:
    config = None

    def __init__(self):
        self.set_config()

    def set_config(self):
        config = configparser.ConfigParser()
        config_file = Path.home() / ".mozark" / "config"
        config.read(config_file)
        api_url = config.get("default", "MOZARK_APP_TESTING_URL")
        username = config.get("default", "MOZARK_APP_TESTING_USERNAME")
        password = config.get("default", "MOZARK_APP_TESTING_PASSWORD")
        client_id = config.get("default", "MOZARK_APP_TESTING_CLIENTID")
        base_download_dir = config.get("default", "BASE_DOWNLOAD_DIR")
        config = {"username": username, "password": password, "api_url": api_url, "client_id": client_id,
                  "base_download_dir": base_download_dir}
        self.config = config

    def get_config(self):
        return self.config

    def login(self):
        user = User(self)
        api_access_token = user.login()
        self.config["api_access_token"] = api_access_token

    def logout(self):
        self.config["api_access_token"] = ""

    # Project

    def create_project(self, project_name=None, project_description=None):
        """Create a new project
        Args:
            project_name (str): unique project name
            project_description (str): short description for a project

        Returns:
            message (str): 'Success' if successful, 'Failure' along with failure reason

            "Failure: Project with `{project_name}` already exists." - in case if project with the given name already exists
        """
        project = Project(self)
        status_message = project.create_project(project_name, project_description)
        return status_message

    def get_project_info(self, project_name=None):
        """ Returns project information -
        Args:
            project_name (str): project name

        Returns:
            project (dict): contains projectName, projectDescription, and unique ID
            {
                "projectName": "",
                "projectDescription": "",
                "projectUUID": ""
            }

            errorMessage (str): 'Failure: Project with name `{project_name}` not found.'
        """
        project = Project(self)
        status_message = project.get_project_info(project_name=project_name)
        return status_message

    def delete_project(self, project_name=None):
        """ Delete project with given project name

        Args:
            project_name (str): project name

        Returns:
            message (str): 'Success' if successful, 'Failure' along with failure reason
        """
        project = Project(self)
        status_message = project.delete_project(project_name=project_name)
        return status_message

    def get_project_list(self):
        """ Returns list of all projects

        Returns:
            projects (list): list of project dict

            [
                {
                    "projectName": "abc",
                    "projectDescription" : "abc desc",
                    "projectUUID": "aabbcc"
                },
                {
                    "projectName": "xyz",
                    "projectDescription": "xyz desc",
                    "projectUUID": "aabbcd"
                }
            ]

            errorMessage (str): "Failure: Project list is empty."

        """
        project = Project(client=self)
        status_message = project.get_project_list()
        return status_message

    # Application

    def upload_application(self, file_category=None, project_name=None, file_path=None):
        """ Upload android or ios application(.apk or .ipa) from given file path

        Args:
            file_category (str): Mandatory 'android-application' or 'ios-application'
            project_name (str): Container project for the application
            file_path (str): relative or absolute path of the file

        Returns:
            message (str): 'Success' if uploaded successfully, 'Failure' along with failure reason

            "Success: File `" + file_name + "` uploaded successfully."
            "Failure: File `" + file_name + "` not uploaded."
            "Error: File `" + file_name + "` already exists."
            "File Not Found Error: File `" + file_name + "` File not found or wrong path."

        """
        file = File(client=self)
        status = file.upload_application(file_category=file_category, project_name=project_name, file_path=file_path)
        return status

    def get_application_info(self, file_name=None):
        """ Returns file information

        Args:
            file_name (str): unique file name

        Returns:
            fileinfo (dict): dictionary containing the metadata about the file

            {
                "fileName" : "",
                "fileCategory": "android-application | ios-application",
                "md5": "",
                "fileURL": "",
                "fileUUID": "",
                "packageName": ""
            }

            errorMessage (str): "Failure: File with name `{file_name}` not found."

            Note:
                "packageName" is present in case of 'android-application'
        """
        file = File(client=self)
        app_list = file.get_application_info(file_name=file_name)
        return app_list

    def delete_application(self, file_name=None):
        """ Delete application with given file name
        Args:
            file_name (str): unique file name

        Returns:
            "Success: File `" + file_name + "` deleted successfully."
            "Failure: File `" + file_name + "` not deleted."
        """
        file = File(client=self)
        status = file.delete_file(file_name=file_name)
        return status

    def get_application_list(self, file_category=None, project_name=None):
        """ Returns list of all application file information

        Args:
            file_category (str): mandatory 'android-application' or 'ios-application'
            project_name (str): optional project_name to filter the android application files

        Returns:
            fileinfo (list): dictionary containing the metadata about the file

            [
                {
                    "fileName" : "",
                    "fileCategory": "android-application",
                    "md5": "",
                    "fileURL": "",
                    "fileUUID": "",
                    "packageName": "",
                    "projectName": ""
                },
                {
                    "fileName" : "",
                    "fileCategory": "ios-application",
                    "md5": "",
                    "fileURL": "",
                    "fileUUID": "",
                    "projectName": ""
                }
            ]

            Note: "projectName" will be present in response if passed in a filter, else the value will be ""
        """
        file = File(client=self)
        app_list = file.get_application_list(file_category=file_category, project_name=project_name)
        return app_list

    def get_application_list_all(self):
        """ Returns list of all application file information

        Args:
            file_category (str): mandatory 'android-application' or 'ios-application'
            project_name (str): optional project_name to filter the android application files

        Returns:
            fileinfo (list): dictionary containing the metadata about the file

            [
                {
                    "fileName" : "",
                    "fileCategory": "android-application",
                    "md5": "",
                    "fileURL": "",
                    "fileUUID": "",
                    "packageName": "",
                    "projectName": ""
                },
                {
                    "fileName" : "",
                    "fileCategory": "ios-application",
                    "md5": "",
                    "fileURL": "",
                    "fileUUID": "",
                    "projectName": ""
                }
            ]

            Note: "projectName" will be present in response if passed in a filter, else the value will be ""
        """
        file = File(client=self)
        app_list = file.get_application_list_all()
        return app_list

    # Native Test Application

    def upload_native_test_application(self, file_category=None, project_name=None, file_path=None):
        """ Upload native test application(.apk or .ipa) from given file path

        Args:
            file_category (str): Mandatory 'android-application' or 'ios-application'
            project_name (str): Container project for the application
            file_path (str): relative or absolute path of the file

        Returns:
            message (str): 'Success' if uploaded successfully, 'Failure' along with failure reason

            "Success: File `" + file_name + "` uploaded successfully."
            "Failure: File `" + file_name + "` not uploaded."
            "Error: File `" + file_name + "` already exists."
        """
        file = File(client=self)
        status = file.upload_native_test_application(file_category=file_category,
                                                     project_name=project_name,
                                                     file_path=file_path)
        return status

    def get_native_test_application_info(self, file_name=None):
        """ Returns file information

        Args:
            file_name (str): unique file name

        Returns:
            fileinfo (dict): dictionary containing the metadata about the file

            {
                "fileName" : "",
                "fileCategory": "android-test-application | ios-test-application",
                "md5": "",
                "fileURL": "",
                "fileUUID": "",
                "testCodePackageName": "",
                "testRunnerName": "",
                "XCTestRunFileUrl": ""
            }

            errorMessage (str): "Failure: File with name `{file_name}` not found."

            Note:
                "testCodePackageName" and "testRunnerName" is present in case of 'android-test-application'
                "XCTestRunFileUrl" is present in case of 'ios-test-application'

        """
        file = File(client=self)
        app_list = file.get_native_test_application_info(file_name=file_name)
        return app_list

    def delete_native_test_application(self, file_name=None):
        """ Delete native test application with given file name
        Args:
            file_name (str): unique file name

        Returns:
            "Success: File `" + file_name + "` deleted successfully."
            "Failure: File `" + file_name + "` not deleted."
        """
        file = File(client=self)
        status = file.delete_file(file_name=file_name)
        return status

    def get_native_test_application_list(self, file_category=None, project_name=None):
        """ Returns list of all application file information

        Args:
            file_category (str): mandatory 'android-application' or 'ios-application'
            project_name (str): optional project_name to filter the android application files

        Returns:
            fileinfo (list): dictionary containing the metadata about the file

            [
                {
                    "fileName" : "",
                    "fileCategory": "android-test-application",
                    "md5": "",
                    "fileURL": "",
                    "fileUUID": "",
                    "testCodePackageName": "",
                    "testRunnerName": "",
                    "projectName": ""
                },
                {
                    "fileName" : "",
                    "fileCategory": "ios-test-application",
                    "md5": "",
                    "fileURL": "",
                    "fileUUID": "",
                    "XCTestRunFileUrl": ""
                    "projectName": ""
                }
            ]

            Note: "projectName" will be present in response if passed in a filter, else the value will be ""
        """
        file = File(client=self)
        app_list = file.get_native_test_application_list(file_category=file_category, project_name=project_name)
        return app_list

    # Device
    def add_device(self, device_parameter=None):
        """ Add Devices

                Args:
                    device_parameter(json): device details

                Returns:
                    message (str): 'Success'

                    "Failure: Device with name " + device_parameter["serial"] + " already exists."
                """
        device = Device(client=self)
        device_added = device.add_device(device_parameter=device_parameter)
        return device_added

    def get_device_list(self, platform=None, device_serial=None):
        """ Returns device information of all devices for a given platform or device_serial
        Args:
            platform (str): device platform type: 'android', 'ios', or 'living-room'
            device_serial (str): unique device serial

        Returns:
            device_info (list): list of dictionary containing the information about the device

            [
                {
                    "deviceSerial": "",
                    "deviceBrand": "",
                    "deviceCity": "",
                    "deviceCountry": "",
                    "deviceModelName": "",
                    "deviceModelNumber": "",
                    "devicePlatform": "android",
                    "deviceOSVersion": "",
                    "deviceSDKVersion": ["", ""],
                    "deviceUUID": "",
                    "deviceNetwork": ""
                },
                {
                    "deviceSerial": "",
                    "deviceBrand": "",
                    "deviceCity": "",
                    "deviceCountry": "",
                    "deviceModelName": "",
                    "deviceModelNumber": "",
                    "devicePlatform": "ios",
                    "deviceOSVersion": "",
                    "deviceSDKVersion": ["", ""],
                    "deviceUUID": "",
                    "deviceNetwork": ""
                },
                {
                    "deviceSerial": "",
                    "deviceBrand": "",
                    "deviceCity": "",
                    "deviceCountry": "",
                    "deviceModelName": "",
                    "deviceModelNumber": "",
                    "devicePlatform": "living-room",
                    "deviceOSVersion": "",
                    "deviceSDKVersion": ["", ""],
                    "deviceUUID": "",
                    "deviceNetwork": ""
                }
            ]
        """
        device = Device(client=self)
        device_list = device.get_devices(platform=platform, device_serial=device_serial)
        return device_list

    # def get_device_busy_slots(self, devices=None):
    #     pass

    def create_tray(self, platform=None, tray_name=None, device_list=None):
        """ Create tray for a given device platform category

        Args:
            platform(str): type of devices
            tray_name (str): Unique tray name(without spaces)
            device_list (list): device list

        Returns:
            message (str): 'Success' if tray is created successfully, 'Failure' along with failure reason

            "Failure: Tray with `{tray_name}` already exists." - in case if a tray with a given name already exists
        """
        tray = Tray(client=self)
        status = tray.create_tray(platform=platform, tray_name=tray_name, device_list=device_list)
        return status

    def get_tray_info(self, tray_name=None):
        """ Returns tray information for a given tray name
        Args:
            tray_name (str): Tray Name

        Returns:

            tray_info(dict): tray information along with device info

            {
                "trayName": "",
                "trayPlatform": "android",
                "trayUUID": "",
                "trayDevices" : [
                    {
                        "deviceSerial": "",
                        "deviceBrand": "",
                        "deviceCity": "",
                        "deviceCountry": "",
                        "deviceModelName": "",
                        "deviceModelNumber": "",
                        "devicePlatform": "android",
                        "deviceOSVersion": "",
                        "deviceSDKVersion": ["", ""],
                        "deviceUUID": "",
                        "deviceNetwork": ""
                    },
                    {
                        "deviceSerial": "",
                        "deviceBrand": "",
                        "deviceCity": "",
                        "deviceCountry": "",
                        "deviceModelName": "",
                        "deviceModelNumber": "",
                        "devicePlatform": "android",
                        "deviceOSVersion": "",
                        "deviceSDKVersion": ["", ""],
                        "deviceUUID": "",
                        "deviceNetwork": ""
                    }
                ]
            }
        """
        tray = Tray(client=self)
        tray_info = tray.get_tray_info(tray_name=tray_name)
        return tray_info

    def update_tray(self, tray_name=None, device_list=None):
        """ Update List of devices for a given tray name

        Args:
            tray_name(str): Tray name
            device_list(list): Updated device list

        Returns:
            message (str): 'Success' if tray is updated successfully, 'Failure' along with failure reason

            "Failure" in case of failure
        """
        tray = Tray(client=self)
        status = tray.update_tray(tray_name=tray_name, device_list=device_list)
        return status

    def delete_tray(self, tray_name=None):
        """ Delete tray with a given tray name

        Args:
            tray_name(str): Tray name

        Returns:
            message (str): 'Success' if tray is updated successfully, 'Failure' along with failure reason

            "Failure" in case of failure
        """
        tray = Tray(client=self)
        status = tray.delete_tray(tray_name=tray_name)
        return status

    def get_tray_list(self):
        """ Get tray list
        Returns:
            tray_list(list): List of tray info

            [
                {
                    "trayName": "",
                    "trayPlatform": "android",
                    "trayUUID": ""
                },
                {
                    "trayName": "",
                    "trayPlatform": "ios",
                    "trayUUID": ""
                },
                {
                    "trayName": "",
                    "trayPlatform": "living-room",
                    "trayUUID": ""
                }
            ]
        """
        tray = Tray(client=self)
        tray_list = tray.get_tray_list()
        return tray_list

    # Test Configuration & Test Parameters

    def get_supported_test_configuration(self, platform=None):
        mobile_test_configuration = {
            "captureHAR": False,
            "captureCPUMetrics": False,
            "captureMemoryMetrics": False,
            "captureBatteryMetrics": False,
            "captureGraphicsMetrics": False,
            "captureDeviceScreenShots": False,
            "recordDeviceScreen": False,
            "captureDeviceNetworkPackets": False
        }
        living_room_test_configuration = {
            "captureDeviceScreenShots": False,
            "recordDeviceScreen": False
        }
        config = {}
        if platform == "android":
            config = mobile_test_configuration
        elif platform == "ios":
            config = mobile_test_configuration
        elif platform == "living-room":
            config = living_room_test_configuration
        return config

    def get_default_test_parameters(self):
        test_parameters = {
            "maxTestDuration": 5,
            "testFramework": "android-uiautomator",
            "projectName": "",
        }
        return test_parameters

    def get_mandatory_test_parameters(self, platform=None):
        pass

    # Test Execution

    def start_test_execution(self,
                             project_name=None,
                             test_framework=None,
                             application_file_name=None,
                             test_application_file_name=None,
                             devices=None,
                             test_configuration=None,
                             test_parameters=None
                             ):
        """ Execute test now for a given configuration

        Args:
            project_name(str): project name
            test_framework(str): supported test framework 'android-uiautomator' or 'ios-xcuitest'
            application_file_name(str): file name of an application .apk or .ipa
            test_application_file_name: file name of a test application .apk or .ipa
            devices(list): list of device serial
            test_configuration(dict): test configuration as a key value pairs
            test_parameters(dict): test parameters as a key value pairs

        Returns:
            test_status(dict): test status containing the message and a test id to monitor

            {
                "message": "Success: Executed/Scheduled successfully",
                "testId": ""
            }
        """
        action = TestExecute(client=self)
        status = action.execute_test_now(project_name=project_name,
                                         test_framework=test_framework,
                                         application_file_name=application_file_name,
                                         test_application_file_name=test_application_file_name,
                                         devices=devices,
                                         test_configuration=test_configuration,
                                         test_parameters=test_parameters
                                         )
        return status

    def get_test_info(self, test_id=None):
        """ get test info

        Args:
            test_id(str):

        Returns:
            {
                "projectName" : "",
                "testFramework": "android-uiautomator | ios-xcuitest | living-room-automate",
                "applicationFileName": "",
                "testApplicationFileName": "",
                "device": "",
                "testStartTime": "",
                "testEndTime": "",
                "testUUID": "",
                "testStatus": "SCHEDULED | STARTED | COMPLETED | ABORTED | FAILED",
                "testStatusDescription": ""
            }

        """
        action = TestExecute(client=self)
        status = action.get_test_info(test_id=test_id)
        return status

    def abort_test_execution(self, test_id=None):
        """ Abort a test which is running

        Args:
            test_id(str):

        Returns:
            testStatus(str): 'Success' if aborted successfully. 'Failure' with failure reason
        """
        action = TestExecute(client=self)
        status = action.abort_test(test_id=test_id)
        return status

    def schedule_test_execution(self,
                                project_name=None,
                                test_framework=None,
                                application_file_name=None,
                                test_application_file_name=None,
                                devices=None,
                                test_configuration=None,
                                test_parameters=None,
                                start_date_time=None,
                                end_date_time=None,
                                interval=None
                                ):
        """ Execute test now for a given configuration

        Args:
            project_name(str): project name
            test_framework(str): supported test framework 'android-uiautomator' or 'ios-xcuitest'
            application_file_name(str): file name of an application .apk or .ipa
            test_application_file_name: file name of a test application .apk or .ipa
            devices(list): list of device serial
            test_configuration(dict): test configuration as a key value pairs
            test_parameters(dict): test parameters as a key value pairs
            start_date_time(datetime): schedule start date and time
            end_date_time(datetime): schedule end date and time
            interval(number): interval between two test runs

        Returns:
            test_status(dict): test status containing the message and a test id to monitor

            {
                "message": "Success: Executed/Scheduled successfully",
                "scheduleUUID": "",
                "testRuns": [
                    {
                        "testUUID" : "",
                        "testStartDateTime": "",
                    },
                    {
                        "testUUID" : "",
                        "testStartDateTime": "",
                    }
                ]
            }

        """
        action = TestExecute(client=self)
        status = action.schedule_test_executions(project_name=project_name,
                                                 test_framework=test_framework,
                                                 application_file_name=application_file_name,
                                                 test_application_file_name=test_application_file_name,
                                                 devices=devices,
                                                 test_configuration=test_configuration,
                                                 test_parameters=test_parameters,
                                                 start_date_time=start_date_time,
                                                 end_date_time=end_date_time,
                                                 interval=interval
                                                 )
        return status

    def update_schedule(self, data=None, schedule_id=None):
        action = TestExecute(client=self)
        """ Update schedule info

                Args:
                    schedule_id(str): schedule id
                    data(json): parameter

                Returns:
                    schedule_update (dict): Schedule Update Successfully
        """
        status = action.update_schedule(data=data, schedule_id=schedule_id)
        return status

    def update_test(self, data=None,test_id=None):
        action = TestExecute(client=self)
        """ Update test info

                Args:
                    test_id(str): test id
                    data(json): parameter
                    
                Returns:
                    test_update (dict): Test Update Successfully
        """
        status = action.update_test(data=data, test_id=test_id)
        return status

    def get_test_schedule_info(self, schedule_id=None):
        """ get test schedule info

        Args:
            schedule_id(str): schedule id

        Returns:
            schedule_info (dict): schedule information containing the list of test info

            {
                "scheduleUUID": "",
                "scheduleStartTime": "",
                "scheduleEndTime": "",
                "testInterval": "",
                "testConfiguration": {}
                "testParameters": {},
                "projectName" : "",
                "testFramework": "android-uiautomator | ios-xcuitest | living-room-automate",
                "applicationFileName": "",
                "testApplicationFileName": "",
                "testInfo": [
                    {
                        "device": "",
                        "testStartTime": "",
                        "testEndTime": "",
                        "testUUID": "",
                        "testStatus": "SCHEDULED | STARTED | COMPLETED | ABORTED | FAILED",
                        "testStatusDescription": ""
                    },
                    {
                        "device": "",
                        "testStartTime": "",
                        "testEndTime": "",
                        "testUUID": "",
                        "testStatus": "SCHEDULED | STARTED | COMPLETED | ABORTED | FAILED",
                        "testStatusDescription": ""
                    }
            }
        """
        action = TestExecute(client=self)
        status = action.get_test_schedule_info(schedule_id=schedule_id)
        return status

    def delete_test_schedule(self, schedule_id=None):
        """ Delete the test schedule

        Args:
            schedule_id(str):

        Returns:
            testScheduleStatus(str): 'Success' if deleted successfully. 'Failure' with failure reason
        """
        action = TestExecute(client=self)
        status = action.delete_schedule(schedule_id=schedule_id)
        return status

    def get_test_schedule_list(self, from_date_time=None, to_date_time=None):
        """ get list of test schedule info

        Returns:
            schedule_info(list): list of schedule info

            [
                {
                    "scheduleUUID": "",
                    "scheduleStartTime": "",
                    "scheduleEndTime": "",
                    "testInterval": "",
                    "testConfiguration": {}
                    "testParameters": {},
                    "projectName" : "",
                    "testFramework": "android-uiautomator | ios-xcuitest | living-room-automate",
                    "applicationFileName": "",
                    "testApplicationFileName": "",
                    "testInfo": [
                        {
                            "device": "",
                            "testStartTime": "",
                            "testEndTime": "",
                            "testUUID": "",
                            "testStatus": "SCHEDULED | STARTED | COMPLETED | ABORTED | FAILED",
                            "testStatusDescription": ""
                        },
                        {
                            "device": "",
                            "testStartTime": "",
                            "testEndTime": "",
                            "testUUID": "",
                            "testStatus": "SCHEDULED | STARTED | COMPLETED | ABORTED | FAILED",
                            "testStatusDescription": ""
                        }
                },
                {
                    "scheduleUUID": "",
                    "scheduleStartTime": "",
                    "scheduleEndTime": "",
                    "testInterval": "",
                    "testConfiguration": {}
                    "testParameters": {},
                    "projectName" : "",
                    "testFramework": "android-uiautomator | ios-xcuitest | living-room-automate",
                    "applicationFileName": "",
                    "testApplicationFileName": "",
                    "testInfo": [
                        {
                            "device": "",
                            "testStartTime": "",
                            "testEndTime": "",
                            "testUUID": "",
                            "testStatus": "SCHEDULED | STARTED | COMPLETED | ABORTED | FAILED",
                            "testStatusDescription": ""
                        },
                        {
                            "device": "",
                            "testStartTime": "",
                            "testEndTime": "",
                            "testUUID": "",
                            "testStatus": "SCHEDULED | STARTED | COMPLETED | ABORTED | FAILED",
                            "testStatusDescription": ""
                        }
                }
            ]
        """
        action = TestExecute(client=self)
        schedule_list = action.get_test_schedule_list(from_date_time=from_date_time, to_date_time=to_date_time)
        return schedule_list

    # Test Analytics
    def get_test_list(self, from_date_time=None, to_date_time=None):
        """ get list of test info

        Returns:
            test_info(list): list of test info

            [
                {
                    "projectName" : "",
                    "testFramework": "android-uiautomator | ios-xcuitest | living-room-automate",
                    "applicationFileName": "",
                    "testApplicationFileName": "",
                    "device": "",
                    "testStartTime": "",
                    "testEndTime": "",
                    "testUUID": "",
                    "testStatus": "SCHEDULED | STARTED | COMPLETED | ABORTED | FAILED",
                    "testStatusDescription": ""
                },
                {
                    "projectName" : "",
                    "testFramework": "android-uiautomator | ios-xcuitest | living-room-automate",
                    "applicationFileName": "",
                    "testApplicationFileName": "",
                    "device": "",
                    "testStartTime": "",
                    "testEndTime": "",
                    "testUUID": "",
                    "testStatus": "SCHEDULED | STARTED | COMPLETED | ABORTED | FAILED",
                    "testStatusDescription": ""
                }
            ]
        """
        analytics = TestAnalytics(client=self)
        test_list = analytics.get_test_list(from_date_time=from_date_time, to_date_time=to_date_time)
        return test_list

    def get_test_execution_info_full(self, test_id=None):
        """ Get test execution info by test id

        Args:
            test_id: test id

        Returns:
            {
                "testUUID": "",
                "testStartDateTime": "",
                "testEndDateTime": "",
                "projectName": "",
                "applicationFileName": "",
                "testApplicationFileName": "",
                "deviceSerial": "",
                "deviceMake": "",
                "deviceModel": "",
                "deviceCity": "",
                "deviceCountry": "",
                "deviceNetwork": "",
                "devicePlatform": "",
                "deviceOSVersion": "",
                "deviceNetworkOperator": "",
                "testStatus": "",
                "testStatusDescription": "",
                "testCasesTotal": "",
                "testCasesPassed": "",
                "testCasesFailed": "",
                "testCases": [
                    {
                        "testCaseName": "",
                        "testCaseResult": "PASS | FAIL",
                        "testCaseStartDateTime": "",
                        "testCaseEndDateTime": "",
                    },
                    {
                        "testCaseName": "",
                        "testCaseResult": "PASS | FAIL",
                        "testCaseStartDateTime": "",
                        "testCaseEndDateTime": "",
                    },
                    {
                        "testCaseName": "",
                        "testCaseResult": "PASS | FAIL",
                        "testCaseStartDateTime": "",
                        "testCaseEndDateTime": "",
                    }
                ],
                "userExperienceKpis": [
                    {
                        "kpiName": "",
                        "kpiValue": "",
                        "testCaseName": ""
                    },
                    {
                        "kpiName": "",
                        "kpiValue": "",
                        "testCaseName": ""
                    },
                    {
                        "kpiName": "",
                        "kpiValue": "",
                        "testCaseName": ""
                    }
                ],
                "events": [
                    {
                        "eventName": "",
                        "eventDateTime": "",
                        "testCaseName": ""
                    },
                    {
                        "eventName": "",
                        "eventDateTime": "",
                        "testCaseName": ""
                    },
                    {
                        "eventName": "",
                        "eventDateTime": "",
                        "testCaseName": ""
                    }
                ]
            }
        """
        analytics = TestAnalytics(client=self)
        result = analytics.get_test_execution_info_full(test_id=test_id)
        return result

    def get_test_execution_info_by_section(self, test_id=None, section=None):
        """ Get Test Execution info by information section

        Args:
            test_id(str): test id
            section(str): supported info section names
                        - 'basic_test_info'
                        - 'test_configuration'
                        - 'test_cases'
                        - 'events'
                        - 'kpis_user_experience'
                        - 'kpis_api_performance_http'
                        - 'files_device_screenshots'
                        - 'files_device_screen_record'
                        - 'files_har'
                        - 'files_device_cpu_metrics'
                        - 'files_device_memory_metrics'
                        - 'files_device_battery_metrics'
                        - 'files_device_graphics_metrics'
                        - 'files_device_network_packets'
                        - 'files_device_debug_logs'
                        - 'files_test_execution_output'
                        - 'files_test_framework_output'
                        - 'kpis_system_performance_cpu_metrics'
                        - 'kpis_system_performance_memory_metrics'
                        - 'kpis_system_performance_battery_metrics'
                        - 'kpis_app_performance_graphics_metrics'
        Returns:
            section_info(dict): section information
        """
        analytics = TestAnalytics(client=self)
        response = analytics.get_test_execution_info_by_section(test_id=test_id, section=section)
        return response

    def download_by_section(self, test_id=None, section=None):
        """ Download test analytics information as a json file, raw file, list of raw files

        Args:
            test_id(str): test id
            section(str): supported info section names
                        - 'basic_test_info'                         - json
                        - 'test_configuration'                      - json
                        - 'test_cases'                              - json
                        - 'events'                                  - json
                        - 'kpis_user_experience'                    - json
                        - 'kpis_api_performance_http'               - json
                        - 'files_device_screenshots'                - png
                        - 'files_device_screen_record'              - mp4
                        - 'files_har'                               - har
                        - 'files_device_cpu_metrics'                - csv
                        - 'files_device_memory_metrics'             - csv
                        - 'files_device_battery_metrics'            - csv
                        - 'files_device_graphics_metrics'           - csv
                        - 'files_device_network_packets'            - pcap
                        - 'files_device_debug_logs'                 - log
                        - 'files_test_execution_output'             - log
                        - 'files_test_framework_output'             - log | xml | json | html
                        - 'kpis_system_performance_cpu_metrics'     - json
                        - 'kpis_system_performance_memory_metrics'  - json
                        - 'kpis_system_performance_battery_metrics' - json
                        - 'kpis_app_performance_graphics_metrics'   - json
        Returns:
            message(str) : 'Success: File filename downloaded successfully.'
                        : 'Failure: Error in downloading file filename.'
        """
        analytics = TestAnalytics(client=self)
        response = analytics.download_by_section(test_id=test_id, section=section)
        return response



import json
from pathlib import Path

import requests


class Device:
    config = None

    def __init__(self, client=None):
        self.config = client.get_config()

    def add_device(self, device_parameter=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}

        if device_parameter["platform"] == "living-room":
            device_parameter["platform"] = "TV"
        # print(device_parameter)
        device_api_url = self.config.get("api_url") + "testexecute/devices"
        response = requests.post(device_api_url, json=device_parameter, headers=new_headers)
        print(response.json())
        try:
            if response.json()["status"] == 200 and response.json()["message"] == "Success":
                return "Success"
            else:
                return "Failure: Device with name " + device_parameter["serial"] + " already exists."
        except KeyError:
            return "Failure: Device with name " + device_parameter["serial"] + " already exists."

    def get_devices(self, platform=None, device_serial=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        if platform == "living-room":
            platform = "TV"
        new_params = {
            "platform": platform,
            "serial": device_serial
        }
        device_api_url = self.config.get("api_url") + "testexecute/devices"
        # Fetch list of devices
        response = requests.get(device_api_url, params=new_params, headers=new_headers)

        device_list = response.json()['data']['list']
        return_message = []
        if len(device_list) > 0:
            for d in device_list:
                if d['platform'] == "TV":
                    device_platform = "living-room"
                else:
                    device_platform = d['platform']
                if device_platform == 'android':
                    file_info = {"deviceSerial": d['serial'],
                                 "deviceBrand": d['brand'],
                                 "deviceCity": d['deviceParameters']['city'],
                                 "deviceCountry": d['deviceParameters']['country'],
                                 "deviceModelName": d['modelName'],
                                 "deviceModelNumber": d['modelNumber'],
                                 "devicePlatform": d['platform'],
                                 "deviceOSVersion": d['osVersion'],
                                 "deviceSDKVersion": d['sdkVersion'],
                                 "deviceUUID": d['uuid'],
                                 "deviceNetwork": d['deviceParameters']['network']
                                 }
                    return_message.append(file_info)
                elif device_platform == 'ios':
                    file_info = {"deviceSerial": d['serial'],
                                 "deviceBrand": d['brand'],
                                 "deviceCity": d['deviceParameters']['city'],
                                 "deviceCountry": d['deviceParameters']['country'],
                                 "deviceModelName": d['modelName'],
                                 "deviceModelNumber": d['modelNumber'],
                                 "devicePlatform": d['platform'],
                                 "deviceOSVersion": d['osVersion'],
                                 "deviceSDKVersion": d['sdkVersion'],
                                 "deviceUUID": d['uuid'],
                                 "deviceNetwork": d['deviceParameters']['network']
                                 }
                    return_message.append(file_info)
                elif device_platform == 'living-room':
                    file_info = {"deviceSerial": d['serial'],
                                 "deviceBrand": d['brand'],
                                 "deviceCity": d['deviceParameters']['city'],
                                 "deviceCountry": d['deviceParameters']['country'],
                                 "deviceModelName": d['modelName'],
                                 "deviceModelNumber": d['modelNumber'],
                                 "devicePlatform": device_platform,
                                 "deviceOSVersion": d['osVersion'],
                                 "deviceSDKVersion": d['sdkVersion'],
                                 "deviceUUID": d['uuid'],
                                 "deviceNetwork": d['deviceParameters']['network']
                                 }
                    return_message.append(file_info)
        return return_message



# from distutils.core import setup
from setuptools import setup

setup(
    name='mozark_sdk',
    packages=['mozark_sdk'],
    version='1.0.2',
    license='MIT',
    description='Automation test APIs',
    author='Mozark',
    author_email='mozark-aws-staging@mozark.ai',
    url='https://mozark.ai',
    # download_url='https://github.com/user/reponame/archive/v_01.tar.gz',  # I explain this later on
    # long_description_content_type = "text/markdown",
    keywords=['MOZARK', 'AUTOMATION', 'EXPERIENCE'],
    install_requires=[
        'configparser==5.3.0',
        'requests==2.28.1'
    ],
    description_file='README.md',
    classifiers=[
        'Development Status :: 3 - Alpha',
        # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Intended Audience :: Developers',  # Define that your audience are developers
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',  # Again, pick a license
        'Programming Language :: Python :: 3',  # Specify which pyhton versions that you want to support
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
)



import json
import requests
from mozark_sdk.device import Device


class Tray:
    config = None

    def __init__(self, client=None):
        self.config = client.get_config()
        self.client = client

    def create_tray(self, platform=None, tray_name=None, device_list=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        data = {
            "trayName": tray_name,
            "devices": device_list,
            "platform": platform
        }
        tray_api_url = self.config.get("api_url") + "tray/create"
        response = requests.post(tray_api_url, json=data, headers=new_headers)
        if response.status_code == 200 and response.json()["body"]["message"] == "Tray Created Successfully":
            return "Success"
        else:
            return "Failure: tray with name " + tray_name + " already exists."

    def get_tray_info(self, tray_name=None):
        device_list = []
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        new_params = {
        }
        tray_api_url = self.config.get("api_url") + "tray/list"
        # Get list of tray created
        response = requests.get(tray_api_url, params=new_params, headers=new_headers)
        devices = {}
        if response.status_code == 200:
            my_resp = json.loads(response.text)
            # print("tray name: ", my_resp['body']['trays'][0]["trayName"])
            for j in range(len(json.loads(response.text)['body']['trays'])):

                if my_resp['body']['trays'][j]["trayName"] == tray_name:
                    for i in range(len(my_resp['body']['trays'][j]["devices"])):
                        device_id = my_resp['body']['trays'][j]["devices"][i]
                        platform = my_resp['body']['trays'][j]["platform"]

                        device_obj = Device(client=self.client)
                        device = device_obj.get_devices(platform=platform, device_serial=device_id)
                        device_list.append(device)

                    tray_info = {
                        "trayName": my_resp['body']['trays'][j]['trayName'],
                        "trayPlatform": my_resp['body']['trays'][j]['platform'],
                        "trayUUID": my_resp['body']['trays'][j]['trayID'],
                        "trayDevices": device_list,
                    }
                    return tray_info
            else:
                return "Failure: tray with name " + tray_name + " not exists."
        else:
            return {"statusCode:": response.status_code, "message": response.text}

    def update_tray(self, tray_name=None, device_list=None):
        # PUT
        # https://hotstar-api.mozark.ai/tray/update?trayid=7eed6665-f470-4b91-81aa-bc399f0f325a
        # {devices: ["RF8R70LNY6W", "RZ8R31QDBZF"]}

        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        tray_id = self.get_trayid_by_trayname(tray_name=tray_name, new_headers=new_headers)
        if tray_id:
            new_params = {
                "trayid": tray_id
            }
            data = {
                "devices": device_list
            }
            tray_api_url = self.config.get("api_url") + "tray/update"
            response = requests.put(tray_api_url, json=data, headers=new_headers, params=new_params)
            if response.status_code == 200:
                my_resp = json.loads(response.text)
                my_resp = my_resp['body']['message']
                return 'Success'
            else:
                return {"statusCode:": response.status_code, "message": response.text}
        else:
            return "Failure: tray with name " + tray_name + " not exists."

    def delete_tray(self, tray_name=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        tray_delete_api_url = self.config.get("api_url") + "tray/delete"
        tray_id = self.get_trayid_by_trayname(tray_name=tray_name, new_headers=new_headers)
        if tray_id:
            new_params = {
                "trayid": tray_id
            }
            # Fetch list of files uploaded
            response = requests.delete(tray_delete_api_url, params=new_params, headers=new_headers)
            if response.status_code == 200:
                my_resp = json.loads(response.text)
                my_resp = my_resp['body']['message']
                return 'Success'
            else:
                return {"statusCode:": response.status_code, "message": response.text}
        else:
            return "Failure: tray with name " + tray_name + " not exists."

    def get_tray_list(self):
        trayinfolist = []
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        new_params = {
        }
        tray_api_url = self.config.get("api_url") + "tray/list"
        # Get list of tray created
        response = requests.get(tray_api_url, params=new_params, headers=new_headers)
        if response.status_code == 200:
            tray_list = json.loads(response.text)
            # my_resp = my_resp['body']['trays']

            for i in range(len(tray_list['body']['trays'])):
                trayname = tray_list['body']['trays'][i]['trayName']
                trayuuid = tray_list['body']['trays'][i]['trayID']
                trayplatform = tray_list['body']['trays'][i]["platform"]

                tray_info = {"trayName": trayname,
                             "trayPlatform": trayplatform,
                             "trayUUID": trayuuid
                             }
                trayinfolist.append(tray_info)
            tray_list = {
                "tray_list": trayinfolist
            }

            return tray_list
        else:
            return {"statusCode:": response.status_code, "message": response.text}

    def get_trayid_by_trayname(self, tray_name=None, new_headers=None):
        tray_list_api_url = self.config.get("api_url") + "tray/list"
        response = requests.get(tray_list_api_url, headers=new_headers)
        if response.status_code == 200:
            my_resp = json.loads(response.text)
            for j in range(len(my_resp['body']['trays'])):
                if my_resp['body']['trays'][j]["trayName"] == tray_name:
                    tray_id = my_resp['body']['trays'][j]["trayID"]
                    return tray_id



import json
import requests
import os
from datetime import datetime


class TestAnalytics:
    config = None

    def __init__(self, client=None):
        self.config = client.get_config()

    def get_test_execution_info_full(self, test_id=None):

        test_info = self.get_test_information(test_id=test_id)
        kpi = self.get_test_kpis(test_id=test_id)
        events = self.get_test_events(test_id=test_id)
        testcases = self.get_test_testcases(test_id=test_id)

        test_information = {
            "test_info": test_info,
            "testCases": testcases,
            "userExperienceKpis": kpi,
            "events": events,
        }

        return test_information

    def is_datetime_in_range(self, start_datetime=None, end_datetime=None, check_datetime=None):
        # format = ['%Y-%m-%dT%H:%M%z', '%Y-%m-%dT%H:%M:%S.%f%z', '%Y-%m-%dT%H:%M:%S.%fZ']
        # format = ['%Y-%m-%dT%H:%M', '%Y-%m-%dT%H:%M:%S', '%Y-%m-%dT%H:%M:%S']
        # if '+05:30' in check_datetime:
        #     check_datetime = check_datetime.split('+')
        #     check_datetime = datetime.strptime(check_datetime[0], format[0])
        # elif '+0530' in check_datetime:
        #     check_datetime = check_datetime.split('.')
        #     check_datetime = datetime.strptime(check_datetime[0], format[1])
        # else:
        #     check_datetime = check_datetime.split('.')
        #     check_datetime = datetime.strptime(check_datetime[0], format[2])
        check_datetime = check_datetime.split('T')
        if not check_datetime[0]:
            return False
        else:
            # print("\n :datetime ",check_datetime)
            check_datetime = datetime.strptime(check_datetime[0], '%Y-%m-%d')
            return start_datetime <= check_datetime <= end_datetime

    def get_test_list(self, from_date_time=None, to_date_time=None):
        test_info_list = []
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        new_params = {
        }
        test_api_url = self.config.get("api_url") + "analytics/tests"
        # Fetch list of tests
        response = requests.get(test_api_url, params=new_params, headers=new_headers)
        if response.status_code == 200:
            test_list = json.loads(response.text)
            test_list = test_list['body']
            for i in range(len(test_list)):
                check_datetime = test_list[i]['testStartTime']
                # check=self.is_datetime_in_range(from_date_time, to_date_time, check_datetime)
                if self.is_datetime_in_range(from_date_time, to_date_time, check_datetime) == True:
                    # print(i, test_list[i])
                    if 'Scheduled' in test_list[i]['testStatus']:
                        testStatus = 'SCHEDULED'
                    elif 'Started' in test_list[i]['testStatus']:
                        testStatus = 'STARTED'
                    elif 'Completed' in test_list[i]['testStatus']:
                        testStatus = 'COMPLETED'
                    elif 'Aborted' in test_list[i]['testStatus']:
                        testStatus = 'ABORTED'
                    else:
                        testStatus = 'FAILED'

                    if '.ipa' in test_list[i]['scriptName']:
                        testFramework = "ios-xcuitest"
                    elif '.apk' in test_list[i]['scriptName']:
                        testFramework = "android-uiautomator"
                    else:
                        testFramework = "living-room-automate"

                    test_info = {
                        "projectName": test_list[i]['projectName'],
                        "testFramework": testFramework,
                        "applicationFileName": test_list[i]['appVersion'],
                        "testApplicationFileName": test_list[i]['scriptName'],
                        "device": test_list[i]['deviceName'],
                        "testStartTime": test_list[i]['testStartTime'],
                        "testEndTime": "",
                        "testUUID": test_list[i]['uuid']['testId'],
                        "testStatus": testStatus,
                        "testStatusDescription": test_list[i]['testStatus']
                    }
                    test_info_list.append(test_info)

            return test_info_list
        else:
            return {"statusCode:": response.status_code, "message": response.text}

    def get_test_information(self, test_id=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        new_params = {
        }
        test_api_url = self.config.get("api_url") + "analytics/tests/" + test_id + "/info"
        # Fetch info of test
        response = requests.get(test_api_url, params=new_params, headers=new_headers)
        if response.status_code == 200:
            test_list = json.loads(response.text)

            try:
                test_info = test_list['body']
                print("\n test config: ", test_info)
                test_information = {
                    "testUUID": test_info['uuid']['testId'],
                    "testStartDateTime": test_info['testStartTime'],
                    "testEndDateTime": test_info['testEndTime'],
                    "projectName": test_info['projectName'],
                    "applicationFileName": test_info['appVersion'],
                    "testApplicationFileName": test_info['scriptName'],
                    "deviceSerial": test_info['deviceSerial'],
                    "deviceMake": "",
                    "deviceModel": test_info['deviceName'],
                    "deviceCity": test_info['deviceLocation'],
                    "deviceCountry": "",
                    "deviceNetwork": test_info['deviceNetwork'],
                    "devicePlatform": test_info['deviceOSVersion'],
                    "deviceOSVersion": test_info['deviceOSVersion'],
                    "deviceNetworkOperator": test_info['operator'],
                    "testStatus": test_info['testStatus'],
                    "testStatusDescription": test_info['testCaseSummary'],
                    "testCasesTotal": test_info['testCaseSummary']['total'],
                    "testCasesPassed": test_info['testCaseSummary']['passed'],
                    "testCasesFailed": test_info['testCaseSummary']['failed'],
                }

                return test_information
            except KeyError:
                return {"statusCode:": response.status_code, "message": f'Nothing in body for test_id {test_id}'}
        else:
            return {"statusCode:": response.status_code, "message": response.text}

    def get_test_configuration(self, test_id=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        new_params = {
        }
        test_api_url = self.config.get("api_url") + "analytics/tests/" + test_id + "/info"
        # Fetch info of test
        response = requests.get(test_api_url, params=new_params, headers=new_headers)
        if response.status_code == 200:
            test_list = json.loads(response.text)

            try:
                testConfiguration = test_list['body']['testConfiguration']
                testConfig = {
                    'testConfiguration': testConfiguration
                }
                return testConfig
            except KeyError:
                return {"statusCode:": response.status_code, "message": f'Nothing in body for test_id {test_id}'}
        else:
            return {"statusCode:": response.status_code, "message": response.text}

    def get_test_testcases(self, test_id=None):
        testcaselist = []
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        new_params = {
        }
        test_api_url = self.config.get("api_url") + "analytics/tests/" + test_id + "/testcases"
        # Fetch info of test
        response = requests.get(test_api_url, params=new_params, headers=new_headers)
        if response.status_code == 200:
            test_list = json.loads(response.text)

            try:
                testcases = test_list['body']
                for i in range(len(testcases['testCases'])):
                    testCaseName = testcases['testCases'][i]['testCaseName']
                    testCaseResult = testcases['testCases'][i]['status']
                    testCaseStartDateTime = ""
                    testCaseEndDateTime = ""

                    testcases_info = {
                        "testCaseName": testCaseName,
                        "testCaseResult": testCaseResult,
                        "testCaseStartDateTime": testCaseStartDateTime,
                        "testCaseEndDateTime": testCaseEndDateTime,
                    }
                    testcaselist.append(testcases_info)
                test_cases = {'test_cases': testcaselist}
                return test_cases
            except KeyError:
                return {"statusCode:": response.status_code, "message": f'Nothing in body for test_id {test_id}'}
        else:
            return {"statusCode:": response.status_code, "message": response.text}

    def get_test_events(self, test_id=None):
        eventexp = []
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        new_params = {
        }
        test_api_url = self.config.get("api_url") + "analytics/tests/" + test_id + "/app/events"
        # Fetch events of test
        response = requests.get(test_api_url, params=new_params, headers=new_headers)
        if response.status_code == 200:
            test_list = json.loads(response.text)
            try:
                events = test_list['body']
                for i in range(len(events["events"])):
                    eventname = events["events"][i]['eventName']
                    eventdatetime = events["events"][i]['time']
                    testcasename = events["events"][i]['testCase']

                    event_info = {
                        "eventName": eventname,
                        "eventDateTime": eventdatetime,
                        "testCaseName": testcasename
                    }
                    eventexp.append(event_info)
                events = {'events': eventexp}
                return events
            except KeyError:
                return {"statusCode:": response.status_code, "message": f'Nothing in body for test_id {test_id}'}
        else:
            return {"statusCode:": response.status_code, "message": response.text}

    def get_test_kpis(self, test_id=None):
        userexperiancekpi = []

        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        new_params = {
        }
        test_api_url = self.config.get("api_url") + "analytics/tests/" + test_id + "/app/kpi/experience"
        # Fetch kpis of test
        response = requests.get(test_api_url, params=new_params, headers=new_headers)
        if response.status_code == 200:
            test_list = json.loads(response.text)
            try:
                kpi = test_list['body']
                for i in range(len(kpi["experience"])):
                    kpiname = kpi["experience"][i]['kpiName']
                    kpivalue = kpi["experience"][i]['value']
                    testcasename = ""

                    kpi_info = {"kpiName": kpiname,
                                "kpiValue": kpivalue,
                                "testCaseName": testcasename
                                }
                    userexperiancekpi.append(kpi_info)
                kpis_user_experience = {'kpis_user_experience': userexperiancekpi}
                return kpis_user_experience
            except KeyError:
                return {"statusCode:": response.status_code, "message": f'Nothing in body for test_id {test_id}'}
        else:
            return {"statusCode:": response.status_code, "message": response.text}

    def get_test_apis(self, test_id=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        new_params = {
        }
        test_api_url = self.config.get("api_url") + "analytics/tests/" + test_id + "/app/resource/httpapi"
        # Fetch http apis of test
        response = requests.get(test_api_url, params=new_params, headers=new_headers)
        if response.status_code == 200:
            test_list = json.loads(response.text)
            try:
                test_list = test_list['body']
                return test_list
            except:
                return {"statusCode:": response.status_code, "message": f'Nothing in body for test_id {test_id}'}
        else:
            return {"statusCode:": response.status_code, "message": response.text}

    def get_test_screenshot_list(self, test_id=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        new_params = {
            "testId": test_id,
            "type": "screenshots"
        }
        test_api_url = self.config.get("api_url") + "testexecute/download"
        # Fetch screenshots of test
        response = requests.get(test_api_url, params=new_params, headers=new_headers)
        if response.status_code == 200:
            test_list = json.loads(response.text)
            try:
                test_list = test_list['data']['list']
            except KeyError:
                return {"statusCode:": response.status_code, "message": f'Nothing in body for test_id {test_id}'}
            return test_list
        else:
            return None

    def get_test_output_file_list(self, test_id=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        new_params = {
            "testId": test_id,
            "type": "output"
        }
        test_api_url = self.config.get("api_url") + "testexecute/download"
        # Fetch screenshots of test
        response = requests.get(test_api_url, params=new_params, headers=new_headers)
        if response.status_code == 200:
            test_list = json.loads(response.text)
            try:
                test_list = test_list['data']['list']
            except KeyError:
                return {"statusCode:": response.status_code, "message": f'Nothing in body for test_id {test_id}'}
            return test_list
        else:
            return {"statusCode:": response.status_code, "message": response.text}

    def download_test_screenshot(self, test_id=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        test_api_url = self.config.get("api_url") + "testexecute/download"
        make_test_id_dir = self.config.get("base_download_dir") + test_id
        if not os.path.exists(make_test_id_dir):
            os.mkdir(make_test_id_dir)
        output_path = make_test_id_dir
        list_screenshots = self.get_test_screenshot_list(test_id=test_id)
        if list_screenshots:
            for i in range(len(list_screenshots)):
                file_name = list_screenshots[i]
                new_params = {
                    "testId": test_id,
                    "type": "screenshots",
                    "fileName": file_name
                }

                # Fetch screenshots of test
                response = requests.get(test_api_url, params=new_params, headers=new_headers)
                if response.status_code == 200:
                    test_list = json.loads(response.text)
                    try:
                        test_list = test_list['data']['list']
                        new_response = requests.get(test_list['url'])
                        file_name = os.path.join(output_path, file_name)
                        open(file_name, "wb").write(new_response.content)
                    except:
                        return {"statusCode:": response.status_code,
                                "message": f'Nothing in body for test_id {test_id}'}
            return f'Success: File downloaded successfully.'
        else:
            return f'Failure: Error in downloading file.'

    def download_test_output_file(self, test_id=None, file_name=None):
        make_test_id_dir = self.config.get("base_download_dir") + test_id
        if not os.path.exists(make_test_id_dir):
            os.mkdir(make_test_id_dir)
        output_path = make_test_id_dir

        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        new_params = {
            "testId": test_id,
            "type": "output",
            "fileName": file_name
        }

        test_api_url = self.config.get("api_url") + "testexecute/download"

        response = requests.get(test_api_url, params=new_params, headers=new_headers)
        if response.status_code == 200:
            test_list = json.loads(response.text)
            try:
                test_list = test_list['data']['list']

                new_response = requests.get(requests.utils.unquote(test_list['url']))

                file_name = os.path.join(output_path, file_name)
                open(file_name, "wb").write(new_response.content)
                return f'Success: File downloaded successfully.'
            except:
                return {"statusCode:": response.status_code, "message": f'Nothing in body for test_id {test_id}'}
        else:
            return f'Failure: Error in downloading file.'

    def download_by_section(self, test_id=None, section=None):

        if section == 'basic_test_info':
            make_json = self.get_test_information(test_id=test_id)
            response = self.create_json(test_id=test_id, section=section, make_json=make_json)

        elif section == 'test_configuration':
            make_json = self.get_test_configuration(test_id=test_id)
            response = self.create_json(test_id=test_id, section=section, make_json=make_json)
        elif section == 'test_cases':
            make_json = self.get_test_testcases(test_id=test_id)
            response = self.create_json(test_id=test_id, section=section, make_json=make_json)

        elif section == 'events':
            make_json = self.get_test_events(test_id=test_id)
            response = self.create_json(test_id=test_id, section=section, make_json=make_json)

        elif section == 'kpis_user_experience':
            make_json = self.get_test_kpis(test_id=test_id)
            response = self.create_json(test_id=test_id, section=section, make_json=make_json)

        # need to handle error if body not present
        elif section == 'kpis_api_performance_http':
            make_json = self.get_test_apis(test_id=test_id)
            print(make_json)
            response = self.create_json(test_id=test_id, section=section, make_json=make_json)

        elif section == 'files_device_screenshots':
            response = self.download_test_screenshot(test_id=test_id)

        elif section == 'kpis_system_performance_cpu_metrics':
            pass
        elif section == 'kpis_system_performance_memory_metrics':
            pass
        elif section == 'kpis_system_performance_battery_metrics':
            pass
        elif section == 'kpis_app_performance_graphics_metrics':
            pass

        else:
            file_lists = self.get_test_output_file_list(test_id=test_id)
            # print("\n file lists: ", file_lists)
            if section == 'files_device_screen_record':
                if 'final_video.mp4' in file_lists:
                    file_name = 'final_video.mp4'
                    response = self.download_test_output_file(test_id=test_id, file_name=file_name)
                else:
                    response = f'Failure: Error in downloading file.'

            elif section == 'files_har':
                if 'har_logs.har' in file_lists:
                    file_name = 'har_logs.har'
                    response = self.download_test_output_file(test_id=test_id, file_name=file_name)
                else:
                    response = f'Failure: Error in downloading file.'

            elif section == 'files_device_cpu_metrics':
                if 'cpu.txt' in file_lists:
                    file_name = 'cpu.txt'
                    response = self.download_test_output_file(test_id=test_id, file_name=file_name)
                else:
                    response = f'Failure: Error in downloading file.'

            elif section == 'files_device_memory_metrics':
                if 'memory.txt' in file_lists:
                    file_name = 'memory.txt'
                    response = self.download_test_output_file(test_id=test_id, file_name=file_name)
                else:
                    response = f'Failure: Error in downloading file.'

            elif section == 'files_device_battery_metrics':
                if 'battery.txt' in file_lists:
                    file_name = 'battery.txt'
                    response = self.download_test_output_file(test_id=test_id, file_name=file_name)
                else:
                    response = f'Failure: Error in downloading file.'

            elif section == 'files_device_graphics_metrics':
                if 'frames.txt' in file_lists:
                    file_name = 'frames.txt'
                    response = self.download_test_output_file(test_id=test_id, file_name=file_name)
                else:
                    response = f'Failure: Error in downloading file.'

            elif section == 'files_device_network_packets':
                if 'packet.pcap' in file_lists:
                    file_name = 'packet.pcap'
                    response = self.download_test_output_file(test_id=test_id, file_name=file_name)
                else:
                    response = f'Failure: Error in downloading file.'

            elif section == 'files_device_debug_logs':
                if 'systemDebugLogs.log' in file_lists:
                    file_name = 'systemDebugLogs.log'
                    response = self.download_test_output_file(test_id=test_id, file_name=file_name)
                else:
                    response = f'Failure: Error in downloading file.'

            elif section == 'files_test_execution_output':
                if 'execution.log' in file_lists:
                    file_name = 'execution.log'
                    response = self.download_test_output_file(test_id=test_id, file_name=file_name)
                else:
                    response = f'Failure: Error in downloading file.'

            # this file not available need error handle
            elif section == 'files_test_framework_output':
                if 'framework.log' in file_lists:
                    file_name = 'framework.log'
                    response = self.download_test_output_file(test_id=test_id, file_name=file_name)
                else:
                    response = f'Failure: Error in downloading file.'
            else:
                response = f'Failure: Error in downloading file.'
        return response

    def get_info_url_for_file(self, test_id=None, file_name=None):
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        new_params = {
            "testId": test_id,
            "type": "output",
            "fileName": file_name
        }

        test_api_url = self.config.get("api_url") + "testexecute/download"

        response = requests.get(test_api_url, params=new_params, headers=new_headers)
        if response.status_code == 200:
            test_list = json.loads(response.text)
            test_list = test_list['data']['list']
            return test_list['url']

    def get_info_url_for_screenshot(self, test_id=None):
        url_list = []
        new_headers = {'Authorization': "Bearer " + self.config.get("api_access_token"),
                       'Content-Type': 'application/json'}
        test_api_url = self.config.get("api_url") + "testexecute/download"

        list_screenshots = self.get_test_screenshot_list(test_id=test_id)
        if list_screenshots:
            for i in range(len(list_screenshots)):
                file_name = list_screenshots[i]
                new_params = {
                    "testId": test_id,
                    "type": "screenshots",
                    "fileName": file_name
                }

                # Fetch screenshots of test
                response = requests.get(test_api_url, params=new_params, headers=new_headers)
                if response.status_code == 200:
                    test_list = json.loads(response.text)
                    try:
                        test_list = test_list['data']['list']
                        url_list.append(test_list['url'])
                    except KeyError:
                        return {"statusCode:": response.status_code, "message": f'Nothing in body for test_id {test_id}'}
            return url_list
        else:
            return f'Failure: Error in information.'

    def get_test_execution_info_by_section(self, test_id=None, section=None):
        if section == 'basic_test_info':
            response = self.get_test_information(test_id=test_id)
        elif section == 'test_configuration':
            response = self.get_test_configuration(test_id=test_id)
        elif section == 'test_cases':
            response = self.get_test_testcases(test_id=test_id)

        elif section == 'events':
            response = self.get_test_events(test_id=test_id)

        elif section == 'kpis_user_experience':
            response = self.get_test_kpis(test_id=test_id)

        # need to handle error if body not present
        elif section == 'kpis_api_performance_http':
            response = self.get_test_apis(test_id=test_id)

        elif section == 'files_device_screenshots':
            response = self.get_info_url_for_screenshot(test_id=test_id)
            response = {"fileURL": response}

        elif section == 'kpis_system_performance_cpu_metrics':
            pass
        elif section == 'kpis_system_performance_memory_metrics':
            pass
        elif section == 'kpis_system_performance_battery_metrics':
            pass
        elif section == 'kpis_app_performance_graphics_metrics':
            pass

        else:
            file_lists = self.get_test_output_file_list(test_id=test_id)
            # print("\n file lists: ", file_lists)
            if section == 'files_device_screen_record':
                if 'final_video.mp4' in file_lists:
                    response = self.get_info_url_for_file(test_id=test_id, file_name='final_video.mp4')
                    response = {"fileURL": response}
                else:
                    response = f'Failure: Error in downloading file.'

            elif section == 'files_har':
                if 'har_logs.har' in file_lists:
                    response = self.get_info_url_for_file(test_id=test_id, file_name='har_logs.har')
                    response = {"fileURL": response}
                else:
                    response = f'Failure: Error in downloading file.'

            elif section == 'files_device_cpu_metrics':
                if 'cpu.txt' in file_lists:
                    response = self.get_info_url_for_file(test_id=test_id, file_name='cpu.txt')
                    response = {"fileURL": response}
                else:
                    response = f'Failure: Error in downloading file.'

            elif section == 'files_device_memory_metrics':
                if 'memory.txt' in file_lists:
                    response = self.get_info_url_for_file(test_id=test_id, file_name='memory.txt')
                    response = {"fileURL": response}
                else:
                    response = f'Failure: Error in downloading file.'

            elif section == 'files_device_battery_metrics':
                if 'battery.txt' in file_lists:
                    response = self.get_info_url_for_file(test_id=test_id, file_name='battery.txt')
                    response = {"fileURL": response}
                else:
                    response = f'Failure: Error in downloading file.'

            elif section == 'files_device_graphics_metrics':
                if 'frames.txt' in file_lists:
                    response = self.get_info_url_for_file(test_id=test_id, file_name='frames.txt')
                    response = {"fileURL": response}
                else:
                    response = f'Failure: Error in downloading file.'

            elif section == 'files_device_network_packets':
                if 'packet.pcap' in file_lists:
                    response = self.get_info_url_for_file(test_id=test_id, file_name='packet.pcap')
                    response = {"fileURL": response}
                else:
                    response = f'Failure: Error in downloading file.'

            elif section == 'files_device_debug_logs':
                if 'systemDebugLogs.log' in file_lists:
                    response = self.get_info_url_for_file(test_id=test_id, file_name='systemDebugLogs.log')
                    response = {"fileURL": response}
                else:
                    response = f'Failure: Error in downloading file.'

            elif section == 'files_test_execution_output':
                if 'execution.log' in file_lists:
                    response = self.get_info_url_for_file(test_id=test_id, file_name='execution.log')
                    response = {"fileURL": response}
                else:
                    response = f'Failure: Error in downloading file.'

            # this file not available need error handle
            elif section == 'files_test_framework_output':
                if 'framework.log' in file_lists:
                    response = self.get_info_url_for_file(test_id=test_id, file_name='framework.log')
                    response = {"fileURL": response}
                else:
                    response = f'Failure: Error in downloading file.'
            else:
                response = f'Failure: Error in downloading file.'
        return response

    def create_json(self, test_id=None, section=None, make_json=None):
        if "statusCode:" not in make_json:
            file_name = self.write_to_file(test_id=test_id, section=section, make_json=make_json)
            response = f'Success: File {file_name} downloaded successfully.'
        else:
            response = f'Failure: Error in downloading file.'
        return response

    def write_to_file(self, test_id=None, section=None, make_json=None):
        make_test_id_dir = self.config.get("base_download_dir") + test_id
        if not os.path.exists(make_test_id_dir):
            os.mkdir(make_test_id_dir)
        output_path = make_test_id_dir
        file_name = f'{output_path}/{section}.json'
        with open(file_name, "w") as outfile:
            outfile.write(json.dumps(make_json))
        return file_name
